#pragma once

#include "../debug/exception.h"

#include <string>

/* cannot_write_data_exception exception class */
class cannot_write_data_exception : public exception::base
{
public:
	cannot_write_data_exception(const std::string& filename)
	{
		this->m_filename = filename;
	}

	virtual std::string to_string(void) const override
	{
		return std::string("Cannot write to file \"") + this->m_filename + std::string("\" !");
	}

private:
	std::string m_filename;
};

/* cannot_read_data_exception exception class */
class cannot_read_data_exception : public exception::base
{
public:
	cannot_read_data_exception(const std::string& filename)
	{
		this->m_filename = filename;
	}

	virtual std::string to_string(void) const override
	{
		return std::string("Cannot read from file \"") + this->m_filename + std::string("\" !");
	}

private:
	std::string m_filename;
};

/* resources_not_found_exception exception class */
class resources_not_found_exception : public exception::base
{
public:
	resources_not_found_exception(const std::string& name)
	{
		this->m_name = name;
	}

	virtual std::string to_string(void) const override
	{
		return "Resources \"" + this->m_name + "\" not found!";
	}

private:
	std::string m_name;
};

/* resource_uninitialized_exception exception class */
class resource_uninitialized_exception : public exception::base
{
public:
	virtual std::string to_string(void) const override
	{
		return "Resource is uninitialized!";
	}
};

/* unknown_resource_type_exception exception class */
class unknown_resource_type_exception : public exception::base
{
public:
	virtual std::string to_string(void) const override
	{
		return "Unknown resource type!";
	}
};

/* invalid_loader_type_exception exception class */
class invalid_loader_type_exception : public exception::base
{
public:
	invalid_loader_type_exception(const std::string& type)
	{
		this->m_type = type;
	}

	virtual std::string to_string(void) const override
	{
		return "Invalid loader \"" + this->m_type + "\" !";
	}

private:
	std::string m_type;
};

/* loader_type_already_exists_exception exception class */
class loader_type_already_exists_exception : public exception::base
{
public:
	loader_type_already_exists_exception(const std::string& type)
	{
		this->m_type = type;
	}

	virtual std::string to_string(void) const override
	{
		return "Loader \"" + this->m_type + "\" already exists !";
	}

private:
	std::string m_type;
};

/* unknown_loader_type_exception exception class */
class unknown_loader_type_exception : public exception::base
{
public:
	unknown_loader_type_exception(const std::string& type)
	{
		this->m_type = type;
	}

	virtual std::string to_string(void) const override
	{
		return "Unknown loader for type \"" + this->m_type + "\" !";
	}

private:
	std::string m_type;
};

/* ill_formed_resource_type_exception exception class */
class ill_formed_resource_type_exception : public exception::base
{
public:
	ill_formed_resource_type_exception(const std::string& handler)
	{
		this->m_handler = handler;
	}

	virtual std::string to_string(void) const override
	{
		return "Ill formed resource \"" + this->m_handler + "\" ! Expecting type:// in front of resource.";
	}

private:
	std::string m_handler;
};