#include <stdio.h>

#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/net/net.h"
#include "../core/instancers/instancer.h"
#include "../core/storage/storage.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"

/* Construct Test */
class construct_test : public test::base
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Construct Test");

		// from string
		{
			const char* resname = "string://10";

			// load resource
			resources::proxy<intval> r(resname);

			// check that it was correctly constructed
			EXPECT_FALSE("Resource shall NOT be pending", r.is_pending());
			EXPECT_NOTNULL("Resource from string shall not be null", r.get());
			EXPECT_INT("Resource from string value shall be 10", r->get(), 10);
			EXPECT_INT("0 resource under watch", get_instance<resources::manager>()->num_watched(), (size_t)0);
		}

		// from base64
		{
			const char* resname = "base64://MTA=";

			// load resource
			resources::proxy<intval> r(resname);

			// check that it was correctly constructed
			EXPECT_FALSE("Resource shall NOT be pending", r.is_pending());
			EXPECT_NOTNULL("Resource from base64 shall not be null", r.get());
			EXPECT_INT("Resource from base64 value shall be 10", r->get(), 10);
			EXPECT_INT("0 resource under watch", get_instance<resources::manager>()->num_watched(), (size_t)0);
		}

		// from file
		{
			const char* resname = "file://test.txt";

			// write file
			{
				FILE* p = nullptr;

				fopen_s(&p, "test.txt", "wb+");

				if (p == nullptr)
					throw_exception(cannot_open_file_exception, "test.txt");

				fputs("10", p);

				fclose(p);
			}

			// load resource
			resources::proxy<intval> r(resname);

			// check that it was correctly constructed
			EXPECT_FALSE("Resource shall NOT be pending", r.is_pending());
			EXPECT_NOTNULL("Resource from file shall not be null", r.get());
			EXPECT_INT("Resource from file value shall be 10", r->get(), 10);
			EXPECT_INT("1 resource under watch", get_instance<resources::manager>()->num_watched(), (size_t)1);
		}

		// invalid file shall trigger exception
		{
			const char* resname = "file://invalid";

			// load resource
			EXPECT_EXCEPTION("Invalid resource name shall trigger exception", [&](void)
				{
					resources::proxy<intval> r(resname);
				});

			EXPECT_INT("0 resource under watch", get_instance<resources::manager>()->num_watched(), (size_t)0);
		}

		// default shall trigger exception
		{
			const char* resname = "invalid";

			// load resource
			EXPECT_EXCEPTION("Undecorated resource name shall trigger exception", [&](void)
				{
					resources::proxy<intval> r(resname);
				});

			EXPECT_INT("0 resource under watch", get_instance<resources::manager>()->num_watched(), (size_t)0);
		}
	}
};

REGISTER_TEST(construct_test);