#include <stdio.h>

#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/net/net.h"
#include "../core/instancers/instancer.h"
#include "../core/storage/storage.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"

/* Exception Test */
class exception_test : public test::base
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Exception Test");

		const char* resname = "file://invalid";

		// synchronous shall trigger exception
		EXPECT_EXCEPTION("Invalid synchronous resource shall trigger exception", [&](void)
			{
				resources::proxy<intval> r(resname);
			});

		// asynchronous shall trigger exception
		EXPECT_NOEXCEPTION("Invalid asynchronous resource shall trigger exception", [&](void)
			{
				resources::async_proxy<intval> r(resname);
			});

		// invalid async resource shall be null
		{
			resources::async_proxy<intval> r(resname);

			EXPECT_NULL("Invalid asynchronous resource shall be null", r.get());
		}

		// uninitialized resource
		{
			resources::proxy<intval> r;

			EXPECT_EXCEPTION("Uninitialized synchronous resource shall trigger exception", [&](void)
				{
					r.get();
				});
		}
	}
};

REGISTER_TEST(exception_test);