#include <stdio.h>

#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/net/net.h"
#include "../core/instancers/instancer.h"
#include "../core/storage/storage.h"
#include "../core/storage/file.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"
#include "sample.h"

/* Storage Test */
class storage_test : public test::base
{
public:
	virtual void run(void) const override
	{
		_comment("Async Resource Storage Test");

		// embedded storage test
		{
			storage::buffer data;

			{
				auto p = std::make_shared<embedded_sample>("string://10");

				storage::container container;
				container.push_back(p);

				data = container.pack('SPC0');
			}

			{
				storage::container container;

				container.unpack(data, [](const std::string& type_name)
					{
						if (!is_type_name<embedded_sample>(type_name))
							throw_exception(unknown_typename_exception, type_name);

						return std::make_shared<embedded_sample>();
					});

				EXPECT_INT("Expect 1 element in container", container.size(), (size_t)1);

				std::string owner, name;
				std::shared_ptr<storage::base> obj;

				std::tie(owner, name, obj) = container.front();

				auto p = std::static_pointer_cast<embedded_sample>(obj);

				EXPECT_NOTNULL("Embedded storage object shall not be null", p);
				EXPECT_INT("Embedded storage resource value shall be 10", p->eval(), 10);
			}
		}

		// derived storage test
		{
			storage::buffer data;

			{
				auto p = std::make_shared<derived_sample>("string://10", 20);

				storage::container container;
				container.push_back(p);

				data = container.pack('SPC0');
			}

			{
				storage::container container;

				container.unpack(data, [](const std::string& type_name)
					{
						if (!is_type_name<derived_sample>(type_name))
							throw_exception(unknown_typename_exception, type_name);

						return std::make_shared<derived_sample>();
					});

				EXPECT_INT("Expect 1 element in container", container.size(), (size_t)1);

				std::string owner, name;
				std::shared_ptr<storage::base> obj;

				std::tie(owner, name, obj) = container.front();

				auto p = std::static_pointer_cast<derived_sample>(obj);

				EXPECT_NOTNULL("Derived storage object shall not be null", p);
				EXPECT_INT("Derived storage resource value shall be 10", p->eval(), 10);
				EXPECT_INT("Derived storage other value shall be 20", p->other(), 20);
			}
		}
	}
};

REGISTER_TEST(storage_test);